package org.hardknockjava.example01;

import java.util.HashSet;
import java.util.Set;

public class LostItem implements Comparable<LostItem> {
    private static Set<LostItem> theSet = new HashSet<LostItem>();

    private Integer theKey = 0;
    private String theString = "";

    public static void main(String[] args) {
        // Add a LostItem to the Set with a key value of 1.
        LostItem theFirstItem = new LostItem(1, "the first one");
        theSet.add(theFirstItem);

        // Report whether the item can be located, and whether
        // it can be found by iterating through the Set.
        report("Added one item to the Set", theFirstItem, theSet);

        // Change the key in our item, then try to find it again.
        theFirstItem.setTheKey(2);
        report("Changed the key; looking for the item", theFirstItem, theSet);

        // Add the revised item to the Set and see what happens.
        theSet.add(theFirstItem);
        report("Added the item again after changing the key", theFirstItem, theSet);

        // Add a new item with the same key as what the first
        // now has and add it to the Set. Since there's already
        // an item with that key in the appropriate hash bucket,
        // the new item shouldn't appear.
        LostItem theSecondItem = new LostItem(theFirstItem.getTheKey(), "the second one");
        theSet.add(theSecondItem);
        report("Tried to add a different item with the same key", theSecondItem, theSet);

        // Remove the item we were unsuccessful at adding, then try again.
        theSet.remove(theSecondItem);
        theSet.add(theSecondItem);
        report("Removed the second item and tried again", theSecondItem, theSet);
    }

    public LostItem(Integer theKey, String theString) {
        super();
        this.theKey = theKey;
        this.theString = theString;
    }

    private static void report(String caption, LostItem theSearchItem, Set<LostItem> theSet) {
        // See if the LostItem provided can be located.
        boolean itemFound = theSet.contains(theSearchItem);
        int theHash = theSearchItem.hashCode();
        System.out.println(caption + "\n" //
                + "Current values for theSearchItem: theKey=" //
                + theSearchItem.getTheKey() + " hashCode = " + theHash //
                + "  theString=" + theSearchItem.getTheString());
        System.out.println("  The key " //
                + (itemFound ? "WAS" : "WAS NOT") //
                + " found in the Set");
        ;

        // Compare theKey to each element in the Set to see
        // if they are, in fact, equal.
        System.out.println("Items in the Set");
        for (LostItem theItem : theSet) {
            boolean equalsTheSearchItem = theItem.equals(theSearchItem);
            System.out.println("  theKey=" + theItem.getTheKey() //
                    + "  theString=" + theItem.getTheString() //
                    + " hashCode=" + theItem.hashCode() //
                    + " equalsTheSearchItem=" + equalsTheSearchItem);
        }
        System.out.println();
    }

    public Integer getTheKey() {
        return theKey;
    }

    public void setTheKey(Integer theKey) {
        this.theKey = theKey;
    }

    public String getTheString() {
        return theString;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((theKey == null) ? 0 : theKey.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (!(obj instanceof LostItem))
            return false;
        LostItem other = (LostItem) obj;
        if (theKey == null) {
            if (other.theKey != null)
                return false;
        } else if (!theKey.equals(other.theKey))
            return false;
        return true;
    }

    @Override
    public int compareTo(LostItem o) {
        return this.getTheKey().compareTo(o.getTheKey());
    }

}
